package bots;

import modules.*;
import modules.states.TemplateStateModule;
import robocode.*;
import utils.BotInfoMap;
import utils.BotMath;

/**
 * Template for mute and talkative robots
 * @author Francesco Baldisserri
 */
public abstract class TemplateBot extends TeamRobot {
	protected EmptyModule[] modules;
	protected double maxDistance;
	protected static final int RADAR=0;
	protected static final int MOVEMENT=1;
	protected static final int GUN=2;
	protected static final int STATE=3;
	protected int count,nextState[];
	protected String nextTarget;
	protected boolean stateSwitched;
	protected BotInfoMap bots;
	protected abstract TemplateStateModule getStateModule();
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		maxDistance=BotMath.pointsDistance(0,0,getBattleFieldWidth(),getBattleFieldHeight());
		modules=new EmptyModule[4];
		bots=new BotInfoMap();
		modules[RADAR]=new RadarModule(this,bots);
		modules[MOVEMENT]=new MovementModule(this,bots);
		modules[GUN]=new GunModule(this,bots);
		modules[STATE]=getStateModule();
		stateSwitched=false;
		while(true){
			if(stateSwitched){
				for(count=0;count<modules.length;count++){
					modules[count].setState(nextState[count]);
					modules[count].setTarget(nextTarget);
				}
				stateSwitched=false;
			}
			for(count=0;count<modules.length;count++)
				modules[count].move();
			execute();
		}
	}
	/**
	 * @return Returns the maxDistance.
	 */
	public double getMaxDistance() {
		return maxDistance;
	}
	/* (non-Javadoc)
	 * @see robocode.Robot#onScannedRobot(robocode.ScannedRobotEvent)
	 */
	public void onScannedRobot(ScannedRobotEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onScannedRobot(arg0);
		execute();
	}
	/**
	 * Set state and target of the robot, possible states are inside TemplateStateManager
	 * @param state State to be set
	 * @param target Target to be set (null if there's not)
	 */
	public void setState(int[] state,String target){
		nextState=state;
		nextTarget=target;
		stateSwitched=true;
		String desc;
		if(nextState==TemplateStateModule.ATTACK_ALL)
			desc="ATTACK_ALL";
		else if(nextState==TemplateStateModule.DEFEND_BOT)
			desc="DEFEND_BOT";
		else if(nextState==TemplateStateModule.ATTACK_BOT)
			desc="ATTACK_BOT";
		else desc="NOT_RECOGNIZED";
		out.println("State:"+desc+"\tTarget:"+nextTarget);
	}
	/* (non-Javadoc)
	 * @see robocode.TeamRobot#onMessageReceived(robocode.MessageEvent)
	 */
	public void onMessageReceived(MessageEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onMessageReceived(arg0);
		execute();
	}
	/* (non-Javadoc)
	 * @see robocode.AdvancedRobot#onCustomEvent(robocode.CustomEvent)
	 */
	public void onCustomEvent(CustomEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onCustomEvent(arg0);
		execute();
	}
	/* (non-Javadoc)
	 * @see robocode.AdvancedRobot#onDeath(robocode.DeathEvent)
	 */
	public void onDeath(DeathEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onDeath(arg0);
		execute();
	}
	/* (non-Javadoc)
	 * @see robocode.AdvancedRobot#onSkippedTurn(robocode.SkippedTurnEvent)
	 */
	public void onSkippedTurn(SkippedTurnEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onSkippedTurn(arg0);
		execute();
	}
	/* (non-Javadoc)
	 * @see robocode.Robot#onBulletHit(robocode.BulletHitEvent)
	 */
	public void onBulletHit(BulletHitEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onBulletHit(arg0);
		execute();
	}
	/* (non-Javadoc)
	 * @see robocode.Robot#onBulletHitBullet(robocode.BulletHitBulletEvent)
	 */
	public void onBulletHitBullet(BulletHitBulletEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onBulletHitBullet(arg0);
		execute();
	}
	/* (non-Javadoc)
	 * @see robocode.Robot#onBulletMissed(robocode.BulletMissedEvent)
	 */
	public void onBulletMissed(BulletMissedEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onBulletMissed(arg0);
		execute();
	}
	/* (non-Javadoc)
	 * @see robocode.Robot#onHitByBullet(robocode.HitByBulletEvent)
	 */
	public void onHitByBullet(HitByBulletEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onHitByBullet(arg0);
		execute();
	}
	/* (non-Javadoc)
	 * @see robocode.Robot#onHitRobot(robocode.HitRobotEvent)
	 */
	public void onHitRobot(HitRobotEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onHitRobot(arg0);
		execute();
	}
	/* (non-Javadoc)
	 * @see robocode.Robot#onHitWall(robocode.HitWallEvent)
	 */
	public void onHitWall(HitWallEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onHitWall(arg0);
		execute();
	}
	/* (non-Javadoc)
	 * @see robocode.Robot#onRobotDeath(robocode.RobotDeathEvent)
	 */
	public void onRobotDeath(RobotDeathEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onRobotDeath(arg0);
		execute();
	}
	/* (non-Javadoc)
	 * @see robocode.Robot#onWin(robocode.WinEvent)
	 */
	public void onWin(WinEvent arg0) {
		for(count=0;count<modules.length;count++)
			modules[count].onWin(arg0);
		execute();
	}
	/**
	 * @return Returns the bots.
	 */
	public BotInfoMap getBots() {
		return bots;
	}
	/**
	 * @return Returns the state.
	 */
	public int[] getState() {
		return nextState;
	}
	/**
	 * @return Returns the target.
	 */
	public String getTarget() {
		return nextTarget;
	}
}