package modules;

import bots.TemplateBot;
import robocode.BulletHitBulletEvent;
import robocode.BulletHitEvent;
import robocode.BulletMissedEvent;
import robocode.CustomEvent;
import robocode.DeathEvent;
import robocode.HitByBulletEvent;
import robocode.HitRobotEvent;
import robocode.HitWallEvent;
import robocode.MessageEvent;
import robocode.RobotDeathEvent;
import robocode.ScannedRobotEvent;
import robocode.SkippedTurnEvent;
import robocode.WinEvent;
import utils.BotInfoMap;

/**
 * Template class for modules
 * @author Francesco Baldisserri
 */
public class EmptyModule {
	/**
	 * No operation state
	 */
	public static final int DO_NOTHING=0;
	/**
	 * Current module state
	 */
	protected int state;
	/**
	 * Current target (if current state needs one)
	 */
	protected String target;
	/**
	 * Bot which owns this module
	 */
	protected TemplateBot owner;
	/**
	 * Contains info on scanned robots;
	 */
	protected BotInfoMap bots;
	/**
	 * Default constructor for EmptyModule
	 * @param owner Bot which owns this module
	 * @param bots BotInfoMap which has all information recorded by bot
	 */
	public EmptyModule(TemplateBot owner,BotInfoMap bots){
		this.owner=owner;
		this.bots=bots;
	}
	/**
	 * Action to execute during robot turn
	 */
	public void move(){}
	/**
	 * Returns the state
	 * @return Returns the state
	 */
	public int getState() {
		return state;
	}
	/**
	 * Sets the state
	 * @param state Sets the state
	 */
	public void setState(int state){
		this.state=state;
	}
	/**
	 * Returns current module target
	 * @return Returns current module target
	 */
	public String getTarget() {
		return target;
	}
	/**
	 * Sets current module target
	 * @param targetName Sets current module target
	 */
	public void setTarget(String targetName) {
		target = targetName;
	}
	/* (non-Javadoc)
	 * @see robocode.AdvancedRobot#onSkippedTurn(robocode.SkippedTurnEvent)
	 */
	public void onSkippedTurn(SkippedTurnEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.TeamRobot#onMessageReceived(robocode.MessageEvent)
	 */
	public void onMessageReceived(MessageEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.AdvancedRobot#onCustomEvent(robocode.CustomEvent)
	 */
	public void onCustomEvent(CustomEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.AdvancedRobot#onDeath(robocode.DeathEvent)
	 */
	public void onDeath(DeathEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.Robot#onBulletHit(robocode.BulletHitEvent)
	 */
	public void onBulletHit(BulletHitEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.Robot#onBulletHitBullet(robocode.BulletHitBulletEvent)
	 */
	public void onBulletHitBullet(BulletHitBulletEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.Robot#onBulletMissed(robocode.BulletMissedEvent)
	 */
	public void onBulletMissed(BulletMissedEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.Robot#onHitByBullet(robocode.HitByBulletEvent)
	 */
	public void onHitByBullet(HitByBulletEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.Robot#onHitRobot(robocode.HitRobotEvent)
	 */
	public void onHitRobot(HitRobotEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.Robot#onHitWall(robocode.HitWallEvent)
	 */
	public void onHitWall(HitWallEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.Robot#onRobotDeath(robocode.RobotDeathEvent)
	 */
	public void onRobotDeath(RobotDeathEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.Robot#onScannedRobot(robocode.ScannedRobotEvent)
	 */
	public void onScannedRobot(ScannedRobotEvent arg0) {}
	/* (non-Javadoc)
	 * @see robocode.Robot#onWin(robocode.WinEvent)
	 */
	public void onWin(WinEvent arg0) {}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return this.getClass().getName()+"\tState: "+state+"\tTarget: "+target;
	}
}