package modules;

import bots.TemplateBot;
import robocode.Bullet;
import robocode.GunTurnCompleteCondition;
import utils.BotInfo;
import utils.BotInfoMap;
import utils.BotMath;

/**
 * Module for gun management
 * @author Francesco Baldisserri
 */
public class GunModule extends EmptyModule {
	/**
	 * Attack bot mode costant
	 */
	public static final int ATTACK_BOT_MODE=1;
	/**
	 * Attack all mode costant
	 */
	public static final int ATTACK_ALL_MODE=2;
	private static final double MIN_FIRE_POWER=0.1;
	private static final double DELTA_TARGETING_TIME=0.1;
	private static final long MAX_INFO_AGE=1;
	private BotInfo bot;
	
	/**
	 * @param owner
	 * @param bots
	 */
	public GunModule(TemplateBot owner,BotInfoMap bots) {
		super(owner,bots);
		setState(ATTACK_ALL_MODE);
		owner.setAdjustGunForRobotTurn(true);
		owner.setAdjustRadarForGunTurn(true);
	}
	/* (non-Javadoc)
	 * @see modules.EmptyModule#move()
	 */
	public void move(){
		bot=null;
		switch(state){
		case(ATTACK_BOT_MODE): 
			if(bots.containsInfo(target))
				bot=bots.getInfo(target);
		break;
		case(ATTACK_ALL_MODE): 
			bot=bots.getNearestBot(owner,true,Double.MAX_VALUE,MAX_INFO_AGE);
		break;
		}
		if(bot!=null) shot(bot);
	}
	/**
	 * @param bot
	 * @return
	 */
	private Bullet shot(BotInfo bot){
		/*owner.out.println("toShot:"+bot);
		owner.out.println("target:"+target);
		owner.out.println("BotsInfoMap:"+bots);*/
		double power=Math.max(Math.min(owner.getMaxDistance()/bot.getDistance(owner),3),MIN_FIRE_POWER);
		double speed=BotMath.shotSpeed(power);
		double effTime=(bot.getDistance(owner)/speed)+owner.getTime();
		double currTime=owner.getTime();
		double myX=owner.getX(),myY=owner.getY();
		double shotTime,enemyX,enemyY,oldDelta,delta=Double.MAX_VALUE;
		do{
			oldDelta=delta;
			shotTime=effTime;
			enemyX=bot.predictX((long)shotTime);
			enemyY=bot.predictY((long)shotTime);
			effTime=BotMath.pointsDistance(myX,myY,enemyX,enemyY)/speed+currTime;
			delta=Math.abs(effTime-shotTime);
			//owner.out.println("Delta:"+delta);
		} while((delta>DELTA_TARGETING_TIME)&&(oldDelta!=delta));
		/*owner.out.println("\nShot: Power="+power+" Speed="+BotMath.shotSpeed(power)+" Distance="+bot.getDistance()+" Time="+time);
		 owner.out.println("PredictX="+x+" PredictY="+y);
		 owner.out.println(bot.toString());*/
		owner.setTurnGunRight(BotMath.normalRelativeAngle(BotMath.getAngle(owner.getX(),owner.getY(),enemyX,enemyY)-owner.getGunHeading()));
		owner.waitFor(new GunTurnCompleteCondition(owner));
		if (owner.getGunHeat() == 0)
			return owner.setFireBullet(power);
		else return null;
	}
}