package modules;

import robocode.HitByBulletEvent;
import robocode.HitRobotEvent;
import robocode.HitWallEvent;
import robocode.TurnCompleteCondition;
import bots.TemplateBot;
import utils.BotInfo;
import utils.BotInfoMap;
import utils.BotMath;

/**
 * Module for movement management
 * @author Francesco Baldisserri
 */
public class MovementModule extends EmptyModule{
	/**
	 * Follow mode state value
	 */
	public static final int FOLLOW_MODE=1;
	/**
	 * Random mode state value
	 */
	public static final int RANDOM_MODE=2;
	private static final double NEAR_DIST_PERC=0.3;
	private static final double BACKWARD_DISTANCE=0.1;
	/**
	 * Default constructor for MovementModule
	 * @param owner Bot which owns this module
	 * @param bots BotInfoMap which has all information recorded by bot
	 */
	public MovementModule(TemplateBot owner,BotInfoMap bots) {
		super(owner,bots);
		setState(RANDOM_MODE);
	}
	/* (non-Javadoc)
	 * @see modules.EmptyModule#move()
	 */
	public void move(){
		switch(state){
		case(RANDOM_MODE):
				owner.setAhead(owner.getMaxDistance()*Math.random());
				owner.setTurnRight(BotMath.normalRelativeAngle(360*Math.random()));
		break;
		case(FOLLOW_MODE):
			if(bots.containsInfo(target))
				goNear(bots.getInfo(target));
		break;
		}
	}
	/**
	 * Goes at a NEAR_DIST_PERC/2*MAX_DISTANCE distance from target
	 * @param bot Target bot
	 */
	private void goNear(BotInfo bot){
		if((bot.getDistance(owner)>NEAR_DIST_PERC*owner.getMaxDistance()))
			goToward(bot.getX(),bot.getY(),bot.getDistance(owner)-NEAR_DIST_PERC/2*owner.getMaxDistance());
	}
	/* (non-Javadoc)
	 * @see modules.EmptyModule#onHitRobot(robocode.HitRobotEvent)
	 */
	public void onHitRobot(HitRobotEvent arg0) {
		super.onHitRobot(arg0);
		if(Math.abs(arg0.getBearing())>90)
			owner.setAhead(owner.getMaxDistance()*BACKWARD_DISTANCE);
		else
			owner.setAhead(-owner.getMaxDistance()*BACKWARD_DISTANCE);
	}
	/* (non-Javadoc)
	 * @see modules.EmptyModule#onHitWall(robocode.HitWallEvent)
	 */
	public void onHitWall(HitWallEvent arg0) {
		super.onHitWall(arg0);
		goToward(owner.getBattleFieldWidth()/2,owner.getBattleFieldHeight()/2,owner.getMaxDistance()*BACKWARD_DISTANCE);
	}
	/* (non-Javadoc)
	 * @see modules.EmptyModule#onHitByBullet(robocode.HitByBulletEvent)
	 */
	public void onHitByBullet(HitByBulletEvent arg0) {
		super.onHitByBullet(arg0);
		owner.setTurnRight(BotMath.normalRelativeAngle(arg0.getBearing()+90));
		if(Math.random()<0.5)
			owner.setAhead(owner.getMaxDistance()*BACKWARD_DISTANCE);
		else 
			owner.setAhead(-owner.getMaxDistance()*BACKWARD_DISTANCE);
	}
	/**
	 * Moves bot toward point (x,y) for a certain distance
	 * @param x X
	 * @param y Y 
	 * @param distance Distance
	 */
	private void goToward(double x,double y,double distance){
		owner.setTurnRight(BotMath.getBearing(owner.getHeading(),owner.getX(),owner.getY(),x,y));
		owner.waitFor(new TurnCompleteCondition(owner));
		owner.setAhead(distance);
	}
}