package modules;

import bots.TemplateBot;
import robocode.RobotDeathEvent;
import robocode.ScannedRobotEvent;
import utils.BotInfo;
import utils.BotInfoMap;
import utils.BotMath;

/**
 * Module for radar management
 * @author Francesco Baldisserri
 */
public class RadarModule extends EmptyModule {
	/**
	 * Keeps the radar on the target
	 */
	public static final int TRACKING_MODE=1;
	/**
	 * Turns the radar in every direction
	 */
	public static final int SCANNING_MODE=2;
	private static final double RADAR_TURNING_RATE=Double.POSITIVE_INFINITY;
	private static final double SWEEP_STEP=45;
	private double direction,sweep;
	
	/**
	 * Default constructor for RadarModule
	 * @param owner Bot which owns this module
	 * @param bots BotInfoMap which has all information recorded by bot
	 */
	public RadarModule(TemplateBot owner,BotInfoMap bots) {
		super(owner,bots);
		setState(SCANNING_MODE);
		direction=1;
		sweep=0;
		owner.setAdjustRadarForRobotTurn(true);
		owner.setAdjustRadarForGunTurn(true);
	}
	public void move(){
		switch(state){
		case(SCANNING_MODE)://radar scanning
			owner.setTurnRadarRight(RADAR_TURNING_RATE*direction); //turns radar
		break;
		case(TRACKING_MODE)://target tracking
			if(bots.containsInfo(target)){//target in bots info 
				BotInfo tar=bots.getInfo(target);
				direction*=-1;
				direction=(BotMath.normalRelativeAngle(tar.predictBearing(owner.getTime(),owner))>0)?1:-1;
				owner.setTurnRadarRight(BotMath.normalRelativeAngle(owner.getHeading()-owner.getRadarHeading()+tar.getBearing(owner)+sweep*direction));
				sweep+=SWEEP_STEP;
			}
			else owner.setTurnRadarRight(RADAR_TURNING_RATE*direction); //turns radar
		break;
		}
	}
	public void onScannedRobot(ScannedRobotEvent arg0) {
		bots.updateInfo(new BotInfo(arg0,owner),owner.getTime());
		sweep=0;
	}
	public void onRobotDeath(RobotDeathEvent arg0) {
		bots.removeInfo(arg0.getName());
	}
}