package modules.states;

import java.io.IOException;
import utils.BotInfo;
import utils.BotInfoMap;
import bots.TalkativeBot;
import bots.TemplateBot;

public class TalkativeStateManager extends TemplateStateModule {
	private String request;
	/**
	 * Default constructor for TalkativeStateManager
	 * @param owner Bot which owns this module
	 * @param bots BotInfoMap which has all information recorded by bot
	 */
	public TalkativeStateManager(TemplateBot owner, BotInfoMap bots) {
		super(owner, bots);
	}
	/**
	 * @param request The request to set.
	 */
	public void setRequest(String request) {
		this.request = request;
	}

	/* (non-Javadoc)
	 * @see modules.EmptyModule#move()
	 */
	public void move() {
		BotInfo bot;
		String target=owner.getTarget();
		int[] state=owner.getState();
		BotInfoMap bots=owner.getBots();
		if(state==ATTACK_ALL){//attack all state
			if(owner.getEnergy()>LOW_ENERGY){
				if(request!=null)//serving help requests
					owner.setState(DEFEND_BOT,request);
					else{//looking for enemies with low energy
						bot=bots.getNearestBot(owner,true,Double.MAX_VALUE,MAX_INFO_AGE);
						if(bot!=null)
							owner.setState(ATTACK_BOT,bot.getName());
					}
				}
			}
		else if(state==ATTACK_BOT){//attack bot state
			if(owner.getEnergy()<=LOW_ENERGY){	//low energy
				try {//posts an help request
					owner.broadcastMessage(new String[]{TalkativeBot.ADD_REQUEST,owner.getName()});
				} catch (IOException e) {
					e.printStackTrace();
				}
				owner.setState(ATTACK_ALL,null);
			}
			else if(request!=null){//serving help requests
				owner.setState(DEFEND_BOT,request);
			}
			else if(!bots.containsInfo(owner.getTarget()))//enemy target dead
				owner.setState(ATTACK_ALL,null);
		}
		else if(state==DEFEND_BOT){//defend bot state
			if(owner.getEnergy()<=LOW_ENERGY){	//own energy low
				try {//posts an help request
					owner.broadcastMessage(new String[]{TalkativeBot.ADD_REQUEST,owner.getName()});
				} catch (IOException e) {
					e.printStackTrace();
				}
				owner.setState(ATTACK_ALL,null);
			}
			else if(!bots.containsInfo(target)){	//friend doesn't need help anymore
				bot=bots.getNearestBot(owner,true,Double.MAX_VALUE,MAX_INFO_AGE);//looking for enemies with low energy
				if(bot!=null)
					owner.setState(ATTACK_BOT,bot.getName());
				else owner.setState(ATTACK_ALL,null);
			}
		}
		else owner.setState(ATTACK_ALL,null);
		request=null;
	}
}