/**
 * 
 */
package utils;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

/**
 * Simple coordination artifact for robot, based on help requests.
 * @author Francesco Baldisserri
 */
public class BotCoordinationArtifact implements Serializable {
	private List pendingRequests,servedRequests;
	/**
	 * Default constructor for BotCoordinationArtifact
	 */
	public BotCoordinationArtifact() {
		pendingRequests = new LinkedList();
		servedRequests = new LinkedList();
	}
	/**
	 * Submit help request
	 * @param name Bot name
	 */
	public void addHelpRequest(String name){
		if(!pendingRequests.contains(name))
			pendingRequests.add(name);
	}
	/**
	 * Ask for help request, if any.
	 * @return Help request, if present.
	 */
	public String serveRequest(){
		String name;
		if(hasRequests()){
			name=(String)pendingRequests.remove(0);
			servedRequests.add(name);
			return name;
		}
		else return null;
	}
	/**
	 * Tells if there are pending requests
	 * @return True if there are pending requests, false otherwise
	 */
	public boolean hasRequests(){
		return (pendingRequests.size()>0);
	}
	/**
	 * Remove any pending or served requests
	 * @param rq Request
	 * @return Returns true if request was present, false otherwise.
	 */
	public boolean removeRequest(String rq){
		return (pendingRequests.remove(rq)||servedRequests.remove(rq));
	}
	/**
	 * Checks that a served request it's still there
	 * @param rq Request
	 * @return True if the requests it's still there, false otherwise.
	 */
	public boolean checkRequest(String rq){
		return servedRequests.contains(rq);
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "Request: Pending:"+pendingRequests.size()+"\tServed:"+servedRequests.size()+"\t"+super.toString();
	}	
}