package utils;

/**
 * Robot math library
 * @author Francesco Baldisserri
 */
public class BotMath {
	/**
	 * Calulate normal relative angle
	 * @param angle Angle to be normalized
	 * @return Returns the angle between -180° and +180°
	 */
	public static final double normalRelativeAngle(double angle) {
		if (angle > -180 && angle <= 180)
			return angle;
		double fixedAngle = angle;
		while (fixedAngle <= -180)
			fixedAngle += 360;
		while (fixedAngle > 180)
			fixedAngle -= 360;
		return fixedAngle;
	}
	/**
	 * Calulate normal absolute angle
	 * @param angle Angle to be normalized
	 * @return Returns the angle between 0° and +360°
	 */
	public static final double normalAbsoluteAngle(double angle){
		if( angle < 0 )         return (360) + (angle % (360));
		else if( angle >= (360) ) return angle % (360);
		else return angle;
	}
	/**
	 * Calculates the bearing of a robot regard a point
	 * @param heading Robot heading
	 * @param x1 Robot X position
	 * @param y1 Robot Y position
	 * @param x2 Target X position
	 * @param y2 Target Y position
	 * @return Returns calculated bearing
	 */
	public static final double getBearing(double heading,double x1,double y1,double x2,double y2){
		return normalRelativeAngle(getAngle(x1,y1,x2,y2)-heading);
	}
	public static final double getAngle(double x1,double y1,double x2,double y2){
		double alfa=Math.atan(Math.abs(x2-x1)/Math.abs(y2-y1));
		if( (x2 < x1) && (y2 < y1) )      { alfa += Math.PI; }
		else if( (x2 >= x1) && (y2 < y1) )  { alfa = Math.PI - alfa; }
		else if( (x2 < x1)  && (y2 >= y1) ) { alfa = 2*Math.PI - alfa; }
		return normalAbsoluteAngle(radians2degree(alfa));
	}
	public static final double radians2degree(double radians){
		return (radians/(2*Math.PI))*360;
	}
	public static final double degree2radians(double degree){
		return (degree/(360))*2*Math.PI;
	}
	public static final double shotSpeed(double power){
		return 20-(3*power);
	}
	public static final double pointsDistance(double x1,double y1,double x2,double y2){
		return Math.sqrt(Math.pow(x2-x1,2)+Math.pow(y2-y1,2));
	}
}